//: /be/optiloading/Cargo.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import java.io.*;

/** 
 * Cargo class used to stow information about the cargo
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class Cargo implements Serializable{
	private float weight;
	private float volume;
	private float specificgravity;
	
	/**
	 * Constructor used to create this object.
	 * Responsible for setting the correct weight, volume and specific gravity.
	 * @param w A floating point value indicating the weight of the cargo in tonnes.
	 * @param s The specific gravity of the cargo.
	 */
	public Cargo(float w, float s){
		volume = w/s;
		weight = w;
		specificgravity = s;
	}
	
	/**
	 * Returns the weight of the cargo in tonnes. The weight is set within the constructor.
	 * @return Floating point value that indicates the weight of the cargo in tonnes.
	 */
	public float getWeight(){
		return weight;
	}
	
	/**
	 * Returns the volume of the cargo in m^3 . The volume is calculated within the constructor.
	 * @return Floating point value that indicates the volume of the cargo in m^3.
	 */
	public float getVolume(){
		return volume;
	}
	
	/**
	 * Returns the specific gravity of the cargo (density relative to fresh water). The specific gravity is set within the constructor.
	 * @return Floating point value that indicates the specific gravity of the cargo.
	 */
	public float getSpecificgravity(){
		return specificgravity;
	}
}