//: be/optiloading/Settings.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.text.*;
import java.text.*;

/** 
 * Settings class creates the settings panel.
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

class Settings extends JFrame{
	private static Settings uniqueInstance;
	private JComboBox creationbox;
	private JComboBox selectionbox;
	private JComboBox crossoverbox;
	private JComboBox mutationbox;
	private JFormattedTextField mutratefield;
	private JFormattedTextField crossratefield;
	private JFormattedTextField popsizefield;
	private JFormattedTextField eliteratefield;
	private JSlider mutrate;
	private JSlider crossrate;
	private JSlider eliterate;
	private JSlider popsize;
	private JCheckBox stability;
	private JCheckBox trim;
	private JCheckBox strength;
	private JSpinner restarts;
	private JSpinner optimum;
	
	/**
	 * Returns the uniqueInstance of the Settings object.
	 * @return Settings object.
	 */
	public static Settings getInstance(){
		if (uniqueInstance == null){
			uniqueInstance = new Settings();
		}
		return uniqueInstance;
	}
	
	//Private constructor (singleton)
	private Settings(){
		setTitle("Settings");
		
		//Genetic Settings
		JPanel geneticsettings = new JPanel();
		geneticsettings.setLayout(new BoxLayout(geneticsettings, BoxLayout.X_AXIS));
		geneticsettings.setBorder(new CompoundBorder(
					 new TitledBorder(new EtchedBorder(),"Genetic Settings"),
				 	 new EmptyBorder(1,5,3,5)));
		
		//Mutation rate
		JPanel mutratepaneltop = new JPanel();
		mutratepaneltop.setLayout(new BorderLayout());
		mutratefield = new JFormattedTextField(NumberFormat.getInstance());
		mutratefield.setToolTipText("Slide the slider to change the mutation rate");
		mutratefield.setBackground(new Color(236,236,236));
		mutratefield.setHorizontalAlignment(JTextField.RIGHT);
		mutratefield.setValue(new Float(0.10));
		mutratefield.setEditable(false);
		mutratefield.setColumns(6);
		mutrate = new JSlider(0,100,10);
		mutrate.setToolTipText("Slide the slider to change the mutation rate");
		mutrate.addChangeListener(new MutRateListener());
		mutratepaneltop.add(BorderLayout.WEST, new JLabel("Mutation rate:"));
		mutratepaneltop.add(BorderLayout.EAST, mutratefield);
		JPanel mutratepanelbottom = new JPanel();
		mutratepanelbottom.setLayout(new BorderLayout());
		mutratepanelbottom.add(BorderLayout.NORTH, mutrate);
		JPanel mutratepanel = new JPanel();
		mutratepanel.setLayout(new BorderLayout());
		mutratepanel.add(BorderLayout.NORTH, mutratepaneltop);
		mutratepanel.add(BorderLayout.SOUTH, mutratepanelbottom);
		
		//Crossover rate
		JPanel crossratepaneltop = new JPanel();
		crossratepaneltop.setLayout(new BorderLayout());
		crossratefield = new JFormattedTextField(NumberFormat.getInstance());
		crossratefield.setToolTipText("Slide the slider to change the crossover rate");
		crossratefield.setBackground(new Color(236,236,236));
		crossratefield.setHorizontalAlignment(JTextField.RIGHT);
		crossratefield.setValue(new Float(0.70));
		crossratefield.setEditable(false);
		crossratefield.setColumns(6);
		crossrate = new JSlider(0,100,70);
		crossrate.setToolTipText("Slide the slider to change the crossover rate");
		crossrate.addChangeListener(new CrossRateListener());
		crossratepaneltop.add(BorderLayout.WEST, new JLabel("Crossover rate:"));
		crossratepaneltop.add(BorderLayout.EAST, crossratefield);
		JPanel crossratepanelbottom = new JPanel();
		crossratepanelbottom.setLayout(new BorderLayout());
		crossratepanelbottom.add(BorderLayout.NORTH, crossrate);
		JPanel crossratepanel = new JPanel();
		crossratepanel.setLayout(new BorderLayout());
		crossratepanel.add(BorderLayout.NORTH, crossratepaneltop);
		crossratepanel.add(BorderLayout.SOUTH, crossratepanelbottom);

		//Population Size
		JPanel popsizepaneltop = new JPanel();
		popsizepaneltop.setLayout(new BorderLayout());
		popsizefield = new JFormattedTextField(NumberFormat.getIntegerInstance());
		popsizefield.setToolTipText("Slide the slider to change the population");
		popsizefield.setBackground(new Color(236,236,236));
		popsizefield.setHorizontalAlignment(JTextField.RIGHT);
		popsizefield.setValue(new Integer(500));
		popsizefield.setEditable(false);
		popsizefield.setColumns(6);
		popsize = new JSlider(2,1000,500);
		popsize.setToolTipText("Slide the slider to change the population size");
		popsize.addChangeListener(new PopSizeListener());
		popsizepaneltop.add(BorderLayout.WEST, new JLabel("Population size:"));
		popsizepaneltop.add(BorderLayout.EAST, popsizefield);
		JPanel popsizepanelbottom = new JPanel();
		popsizepanelbottom.setLayout(new BorderLayout());
		popsizepanelbottom.add(BorderLayout.NORTH, popsize);
		JPanel popsizepanel = new JPanel();
		popsizepanel.setLayout(new BorderLayout());
		popsizepanel.add(BorderLayout.NORTH, popsizepaneltop);
		popsizepanel.add(BorderLayout.SOUTH, popsizepanelbottom);
		
		//Elitism rate
		JPanel eliteratepaneltop = new JPanel();
		eliteratepaneltop.setLayout(new BorderLayout());
		eliteratefield = new JFormattedTextField(NumberFormat.getInstance());
		eliteratefield.setToolTipText("Slide the slider to change the elitism rate");
		eliteratefield.setBackground(new Color(236,236,236));
		eliteratefield.setHorizontalAlignment(JTextField.RIGHT);
		eliteratefield.setValue(new Float(0.02));
		eliteratefield.setEditable(false);
		eliteratefield.setColumns(6);
		eliterate = new JSlider(0,50,2);
		eliterate.setToolTipText("Slide the slider to change the elitism rate");
		eliterate.addChangeListener(new EliteRateListener());
		eliteratepaneltop.add(BorderLayout.WEST, new JLabel("Elitism rate:"));
		eliteratepaneltop.add(BorderLayout.EAST, eliteratefield);
		JPanel eliteratepanelbottom = new JPanel();
		eliteratepanelbottom.setLayout(new BorderLayout());
		eliteratepanelbottom.add(BorderLayout.NORTH, eliterate);
		JPanel eliteratepanel = new JPanel();
		eliteratepanel.setLayout(new BorderLayout());
		eliteratepanel.add(BorderLayout.NORTH, eliteratepaneltop);
		eliteratepanel.add(BorderLayout.SOUTH, eliteratepanelbottom);
		
		//Creation method
		JPanel creationpanel = new JPanel();
		creationpanel.setLayout(new BorderLayout());
		creationpanel.add(BorderLayout.WEST, new JLabel("Creation:"));
		String[] creationlist = {"Random","Full tanks"};
		creationbox = new JComboBox(creationlist);
		creationbox.setSelectedItem("Full tanks");
		creationbox.setToolTipText("Select the creation method");
		creationpanel.add(BorderLayout.EAST, creationbox);
		
		//Selection method
		JPanel selectionpanel = new JPanel();
		selectionpanel.setLayout(new BorderLayout());
		selectionpanel.add(BorderLayout.WEST, new JLabel("Selection:"));
		String[] selectionlist = {"Rank","Roulette"};
		selectionbox = new JComboBox(selectionlist);
		selectionbox.setToolTipText("Select the selection method");
		selectionpanel.add(BorderLayout.EAST, selectionbox);
		
		//Crossover method
		JPanel crossoverpanel = new JPanel();
		crossoverpanel.setLayout(new BorderLayout());
		crossoverpanel.add(BorderLayout.WEST, new JLabel("Crossover:"));
		String[] crossoverlist = {"Mean","One Point","Two Point","Uniform"};
		crossoverbox = new JComboBox(crossoverlist);
		crossoverbox.setSelectedItem("Uniform");
		crossoverbox.setToolTipText("Select the crossover method");
		crossoverpanel.add(BorderLayout.EAST, crossoverbox);
		
		//Mutation method
		JPanel mutationpanel = new JPanel();
		mutationpanel.setLayout(new BorderLayout());
		mutationpanel.add(BorderLayout.WEST, new JLabel("Mutation:"));
		mutationpanel.add(BorderLayout.CENTER, new JLabel("     "));
		String[] mutationlist = {"Switch","Add/subtract random"};
		mutationbox = new JComboBox(mutationlist);
		mutationbox.setToolTipText("Select the mutation method");
		mutationpanel.add(BorderLayout.EAST, mutationbox);
		
		//Genetic rates
		JPanel geneticrates = new JPanel();
		geneticrates.setBorder(new EmptyBorder(3,5,3,17));
		geneticrates.setLayout(new BoxLayout(geneticrates, BoxLayout.Y_AXIS));
		geneticrates.add(mutratepanel);
		geneticrates.add(crossratepanel);
		geneticrates.add(popsizepanel);
		geneticrates.add(eliteratepanel);
		
		//Genetic methods
		JPanel geneticmethods = new JPanel();
		geneticmethods.setBorder(new EmptyBorder(3,10,3,5));
		geneticmethods.setLayout(new BoxLayout(geneticmethods, BoxLayout.Y_AXIS));
		creationbox.setPreferredSize(mutationbox.getPreferredSize());
		crossoverbox.setPreferredSize(mutationbox.getPreferredSize());
		selectionbox.setPreferredSize(mutationbox.getPreferredSize());
		creationpanel.setPreferredSize(mutationpanel.getPreferredSize());
		selectionpanel.setPreferredSize(mutationpanel.getPreferredSize());
		crossoverpanel.setPreferredSize(mutationpanel.getPreferredSize());
		JPanel method1 = new JPanel(new GridBagLayout());
		method1.add(creationpanel,new GridBagConstraints());
		geneticmethods.add(method1);
		JPanel method2 = new JPanel(new GridBagLayout());
		method2.add(selectionpanel,new GridBagConstraints());
		geneticmethods.add(method2);
		JPanel method3 = new JPanel(new GridBagLayout());
		method3.add(crossoverpanel,new GridBagConstraints());
		geneticmethods.add(method3);
		JPanel method4 = new JPanel(new GridBagLayout());
		method4.add(mutationpanel,new GridBagConstraints());
		geneticmethods.add(method4);
		
		//Add to genetic settings panel
		geneticsettings.add(geneticrates);
		geneticsettings.add(new JSeparator(SwingConstants.VERTICAL));
		geneticsettings.add(geneticmethods);
		
		//Criteria
		JPanel criteria = new JPanel();
		criteria.setLayout(new GridLayout(3,0));
		criteria.setBorder(new CompoundBorder(
					 	   new TitledBorder(new EtchedBorder(),"Criteria"),
				 	 	   new EmptyBorder(1,5,3,5)));
		stability = new JCheckBox("Stability",null,true);
		stability.addItemListener(new CriteriaListener());
		stability.setToolTipText("Check/uncheck to add/remove stability criteria");
		trim = new JCheckBox("Trim",null,true);
		trim.addItemListener(new CriteriaListener());
		trim.setToolTipText("Check/uncheck to add/remove trim criteria");
		strength = new JCheckBox("Hull stresses",null,true);
		strength.addItemListener(new CriteriaListener());
		strength.setToolTipText("Check/uncheck to add/remove hull stress criteria");
		criteria.add(stability);
		criteria.add(trim);
		criteria.add(strength);
		
		//Stop condition
		JPanel condition = new JPanel();
		condition.setLayout(new GridLayout(4,0));
		condition.setBorder(new CompoundBorder(
					 	   new TitledBorder(new EtchedBorder(),"Stop Condition"),
				 	 	   new EmptyBorder(1,5,3,5)));
		restarts = new JSpinner(new SpinnerNumberModel(5,1,100,1));
		optimum = new JSpinner(new SpinnerNumberModel(25,5,1000,1));
		condition.add(new JLabel("<html>Number of similar consecutive<br>solutions before restart:</html>"));
		condition.add(optimum);
		condition.add(new JLabel("Number of restarts:"));
		condition.add(restarts);
				
		//Criteria + Stop
		JPanel left = new JPanel();
		left.setLayout(new BoxLayout(left,BoxLayout.Y_AXIS));
		left.add(criteria);
		left.add(condition);
		
		//Add to frame
		getContentPane().add(BorderLayout.CENTER, left);
		getContentPane().add(BorderLayout.EAST, geneticsettings);
		setIconImage(new ImageIcon(this.getClass().getResource("/images/icon.png")).getImage());
		setResizable(false);
		pack();
	}
	
	/**
	 * Overwritten method to make the settingsframe visible/invisible and center the frame
	 * @param b Boolean true is visible, false is invisible
	 */
	public void setVisible(boolean b){
		setLocationRelativeTo(null);
		super.setVisible(b);
	}
	
	// ItemListener that checks that at least one criteria is selected
	class CriteriaListener implements ItemListener{
		public void itemStateChanged(ItemEvent e){
			JCheckBox source = (JCheckBox) e.getSource();
			if((trim.isSelected()==false) && (stability.isSelected()==false) && (strength.isSelected()==false)){
				source.setSelected(true);
				JOptionPane.showMessageDialog(uniqueInstance,"At least one criteria has to be selected.");
			}
		}
	}
	
	// ChangeListener that changes the value of the field when the slider is moved
	class MutRateListener implements ChangeListener {
		public void stateChanged(ChangeEvent e){
			JSlider source = (JSlider)e.getSource();
			float floatvalue = (source.getValue()/100f);
			mutratefield.setValue(floatvalue);
		}
	}
	
	// ChangeListener that changes the value of the field when the slider is moved
	class CrossRateListener implements ChangeListener {
		public void stateChanged(ChangeEvent e){
			JSlider source = (JSlider)e.getSource();
			float floatvalue = (source.getValue()/100f);
			crossratefield.setValue(floatvalue);
		}
	}
	
	// ChangeListener that changes the value of the field when the slider is moved
	class PopSizeListener implements ChangeListener {
		public void stateChanged(ChangeEvent e){
			JSlider source = (JSlider)e.getSource();
			int nvalue;
			if (source.getValue() % 2 != 0){
				nvalue = source.getValue() +1;
			} else {nvalue = source.getValue();}
			popsizefield.setValue(nvalue);
		}
	}
	
	// ChangeListener that changes the value of the field when the slider is moved
	class EliteRateListener implements ChangeListener {
		public void stateChanged(ChangeEvent e){
			JSlider source = (JSlider)e.getSource();
			float floatvalue = (source.getValue()/100f);
			eliteratefield.setValue(floatvalue);
		}
	}
	
	/**
	 * Method to get the crossover rate.
	 * @return Floating point value containing the crossover rate.
	 */
	public float getCrossrate() {
		Number f = (Number) crossratefield.getValue();
		return  f.floatValue(); 
	}
	
	/**
	 * Method to get the mutation rate.
	 * @return Floating point value containing the mutation rate.
	 */
	public float getMutationrate() {
		Number f = (Number) mutratefield.getValue();
		return  f.floatValue();
	}
	
	/**
	 * Method to get the population size.
	 * @return Integer value containing the population size.
	 */
	public int getPopulationsize() {
		return (Integer) popsizefield.getValue();
	}
	
	/**
	 * Method to get the number of restarts.
	 * @return Integer value containing the number of restarts.
	 */
	public int getRestarts(){
		return (Integer) restarts.getValue();
	}
	
	/**
	 * Method to get the number of simular consecutive solutions before restart.
	 * @return Integer value containing the number of simular consecutive solutions before restart.
	 */
	public int getOptimum(){
		return (Integer) optimum.getValue();
	}
	
	/**
	 * Method to get the elitism rate.
	 * @return Integer value containing the elitism rate.
	 */
	public float getEliterate() {
		Number f = (Number) eliteratefield.getValue();
		return  f.floatValue();
	}
	
	/**
	 * Method to get the creation method.
	 * @return Integer value containing the index value of the creation method.
	 */
	public int getCreation() {
		return creationbox.getSelectedIndex();
	}
	
	/**
	 * Method to get the selection method.
	 * @return Integer value containing the index value of the selection method.
	 */
	public int getSelection() {
		return selectionbox.getSelectedIndex();
	}
	
	/**
	 * Method to get the crossover method.
	 * @return Integer value containing the index value of the crossover method.
	 */
	public int getCrossover() {
		return crossoverbox.getSelectedIndex();
	}
	
	/**
	 * Method to get the mutation method.
	 * @return Integer value containing the index value of the mutation method.
	 */
	public int getMutation() {
		return mutationbox.getSelectedIndex();
	}
	
	/**
	 * Method to check if the strength criteria is selected
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isStrength(){
		return strength.isSelected();
	}
	
	/**
	 * Method to check if the trim criteria is selected
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isTrim(){
		return trim.isSelected();
	}
	
	/**
	 * Method to check if the stability criteria is selected
	 * @return Boolean value indication if the checkbox is selected.
	 */
	public boolean isStability(){
		return stability.isSelected();
	}
}