package be.optiloading;

import javax.swing.*;
import javax.swing.text.html.*;
import java.net.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.text.*;
import javax.print.*;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import javax.swing.JEditorPane;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;
import javax.swing.text.View;
import javax.swing.text.html.HTMLDocument;

class TestPrint {
	public static void main(String[] args) {
		new TestPrint();
	}
	
	public TestPrint(){
		JEditorPane html = new JEditorPane();
		html.setContentType("text/html");
		html.setEditable(false);
		try {
        	html.setPage(TestPrint.class.getResource("/help/preface.html"));
        } catch (IOException e) {
			System.err.println("Attempted to read a bad URL");
        }
		JScrollPane scroll = new JScrollPane(html);

		HTMLDocument htmldoc = (HTMLDocument) html.getDocument();
		
		new DocumentRenderer().print(htmldoc);
		
		JFrame frame = new JFrame();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.getContentPane().add(BorderLayout.CENTER,scroll);
		frame.setResizable(false);
		frame.setSize(300,300);
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
	}
	
	private class DocumentRenderer implements Printable {

	  protected int currentPage = -1;
	  protected JEditorPane jeditorPane;
	  protected double pageEndY = 0;
	  protected double pageStartY = 0;
	  protected boolean scaleWidthToFit = true;
	  protected PageFormat pFormat;
	  protected PrinterJob pJob;

	  public DocumentRenderer() {
	    pFormat = new PageFormat();
	    pJob = PrinterJob.getPrinterJob();
	  }

	  public Document getDocument() {
	    if (jeditorPane != null) return jeditorPane.getDocument();
	    else return null;
	  }

	  public boolean getScaleWidthToFit() {
	    return scaleWidthToFit;
	  }

	  public void pageDialog() {
	    pFormat = pJob.pageDialog(pFormat);
	  }

	  public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) {
	    double scale = 1.0;
	    Graphics2D graphics2D;
	    View rootView;
	    graphics2D = (Graphics2D) graphics;
	    jeditorPane.setSize((int) pageFormat.getImageableWidth(),Integer.MAX_VALUE);
	    jeditorPane.validate();
	    rootView = jeditorPane.getUI().getRootView(jeditorPane);
	    if ((scaleWidthToFit) && (jeditorPane.getMinimumSize().getWidth() >
	    pageFormat.getImageableWidth())) {
	      scale = pageFormat.getImageableWidth()/
	      jeditorPane.getMinimumSize().getWidth();
	      graphics2D.scale(scale,scale);
	    }
	    graphics2D.setClip((int) (pageFormat.getImageableX()/scale),
	    (int) (pageFormat.getImageableY()/scale),
	    (int) (pageFormat.getImageableWidth()/scale),
	    (int) (pageFormat.getImageableHeight()/scale));
	    if (pageIndex > currentPage) {
	      currentPage = pageIndex;
	      pageStartY += pageEndY;
	      pageEndY = graphics2D.getClipBounds().getHeight();
	    }
	    graphics2D.translate(graphics2D.getClipBounds().getX(),
	    graphics2D.getClipBounds().getY());
	    Rectangle allocation = new Rectangle(0,
	    (int) -pageStartY,
	    (int) (jeditorPane.getMinimumSize().getWidth()),
	    (int) (jeditorPane.getPreferredSize().getHeight()));
	    if (printView(graphics2D,allocation,rootView)) {
	      return Printable.PAGE_EXISTS;
	    }
	    else {
	      pageStartY = 0;
	      pageEndY = 0;
	      currentPage = -1;
	      return Printable.NO_SUCH_PAGE;
	    }
	  }

	  public void print(HTMLDocument htmlDocument) {
	    setDocument(htmlDocument);
	    printDialog();
	  }

	  public void print(JEditorPane jedPane) {
	    setDocument(jedPane);
	    printDialog();
	  }

	  public void print(PlainDocument plainDocument) {
	    setDocument(plainDocument);
	    printDialog();
	  }

	  protected void printDialog() {
	    if (pJob.printDialog()) {
	      pJob.setPrintable(this,pFormat);
	      try {
	        pJob.print();
	      }
	      catch (PrinterException printerException) {
	        pageStartY = 0;
	        pageEndY = 0;
	        currentPage = -1;
	        System.out.println("Error Printing Document");
	      }
	    }
	  }

	  protected boolean printView(Graphics2D graphics2D, Shape allocation,
	  View view) {
	    boolean pageExists = false;
	    Rectangle clipRectangle = graphics2D.getClipBounds();
	    Shape childAllocation;
	    View childView;

	    if (view.getViewCount() > 0 &&
	          !view.getElement().getName().equalsIgnoreCase("td")) {
	      for (int i = 0; i < view.getViewCount(); i++) {
	        childAllocation = view.getChildAllocation(i,allocation);
	        if (childAllocation != null) {
	          childView = view.getView(i);
	          if (printView(graphics2D,childAllocation,childView)) {
	            pageExists = true;
	          }
	        }
	      }
	    } else {
	      if (allocation.getBounds().getMaxY() >= clipRectangle.getY()) {
	        pageExists = true;
	        if ((allocation.getBounds().getHeight() > clipRectangle.getHeight()) &&
	        (allocation.intersects(clipRectangle))) {
	          view.paint(graphics2D,allocation);
	        } else {
	          if (allocation.getBounds().getY() >= clipRectangle.getY()) {
	            if (allocation.getBounds().getMaxY() <= clipRectangle.getMaxY()) {
	              view.paint(graphics2D,allocation);
	            } else {
	              if (allocation.getBounds().getY() < pageEndY) {
	                pageEndY = allocation.getBounds().getY();
	              }
	            }
	          }
	        }
	      }
	    }
	    return pageExists;
	  }

	  protected void setContentType(String type) {
	    jeditorPane.setContentType(type);
	  }

	  public void setDocument(HTMLDocument htmlDocument) {
	    jeditorPane = new JEditorPane();
	    setDocument("text/html",htmlDocument);
	  }

	  public void setDocument(JEditorPane jedPane) {
	    jeditorPane = new JEditorPane();
	    setDocument(jedPane.getContentType(),jedPane.getDocument());
	  }

	  public void setDocument(PlainDocument plainDocument) {
	    jeditorPane = new JEditorPane();
	    setDocument("text/plain",plainDocument);
	  }

	  protected void setDocument(String type, Document document) {
	    setContentType(type);
	    jeditorPane.setDocument(document);
	  }

	  public void setScaleWidthToFit(boolean scaleWidth) {
	    scaleWidthToFit = scaleWidth;
	  }
	}
}

